@@ -23,9 +23,11 @@ namespace NUnit.Gui
 	using NUnit.UiKit;
 	using CP.Windows.Forms;
 
-	public class NUnitForm : System.Windows.Forms.Form
+	public class NUnitForm : NUnitFormBase
 	{
-		#region Instance variables
+        static Logger log = InternalTrace.GetLogger(typeof(NUnitForm));
+        
+        #region Instance variables
 
 		// Handlers for our recentFiles and recentProjects
 		private RecentFileMenuHandler recentProjectsMenuHandler;
@@ -42,6 +44,9 @@ namespace NUnit.Gui
 		// Our current run command line options
 		private GuiOptions guiOptions;
 
+        // Our 'presenter' - under development
+        private NUnitPresenter presenter;
+
 		private System.ComponentModel.IContainer components;
 
 		private System.Windows.Forms.Panel leftPanel;
@@ -52,10 +57,9 @@ namespace NUnit.Gui
 
 		public System.Windows.Forms.GroupBox groupBox1;
 		public System.Windows.Forms.Button runButton;
-		private System.Windows.Forms.Button stopButton;
-		public System.Windows.Forms.Label suiteName;
+        private System.Windows.Forms.Button stopButton;
 		public NUnit.UiKit.TestProgressBar progressBar;
-		private System.Windows.Forms.Label runCount;
+		private CP.Windows.Forms.ExpandingLabel runCount;
 
 		public NUnit.UiKit.ResultTabs resultTabs;
 
@@ -126,19 +130,23 @@ namespace NUnit.Gui
 		private System.Windows.Forms.MenuItem toolsMenuSeparator1;
 		private System.Windows.Forms.MenuItem assemblyDetailsMenuItem;
         private MenuItem runtimeMenuItem;
+        private MenuItem openLogDirectoryMenuItem;
+        private ExpandingLabel suiteName;
 		private System.Windows.Forms.MenuItem addAssemblyMenuItem;
 
 		#endregion
 		
 		#region Construction and Disposal
 
-		public NUnitForm( GuiOptions guiOptions )
+		public NUnitForm( GuiOptions guiOptions ) : base("NUnit")
 		{
 			InitializeComponent();
 
 			this.guiOptions = guiOptions;
 			this.recentFilesService = Services.RecentFiles;
 			this.userSettings = Services.UserSettings;
+
+            this.presenter = new NUnitPresenter(this, TestLoader);
 		}
 
 		protected override void Dispose( bool disposing )
@@ -216,6 +224,7 @@ namespace NUnit.Gui
             this.assemblyDetailsMenuItem = new System.Windows.Forms.MenuItem();
             this.saveXmlResultsMenuItem = new System.Windows.Forms.MenuItem();
             this.exceptionDetailsMenuItem = new System.Windows.Forms.MenuItem();
+            this.openLogDirectoryMenuItem = new System.Windows.Forms.MenuItem();
             this.toolsMenuSeparator1 = new System.Windows.Forms.MenuItem();
             this.settingsMenuItem = new System.Windows.Forms.MenuItem();
             this.toolsMenuSeparator2 = new System.Windows.Forms.MenuItem();
@@ -227,10 +236,10 @@ namespace NUnit.Gui
             this.treeSplitter = new System.Windows.Forms.Splitter();
             this.rightPanel = new System.Windows.Forms.Panel();
             this.groupBox1 = new System.Windows.Forms.GroupBox();
-            this.runCount = new System.Windows.Forms.Label();
+            this.suiteName = new CP.Windows.Forms.ExpandingLabel();
+            this.runCount = new CP.Windows.Forms.ExpandingLabel();
             this.stopButton = new System.Windows.Forms.Button();
             this.runButton = new System.Windows.Forms.Button();
-            this.suiteName = new System.Windows.Forms.Label();
             this.progressBar = new NUnit.UiKit.TestProgressBar();
             this.resultTabs = new NUnit.UiKit.ResultTabs();
             this.toolTip = new System.Windows.Forms.ToolTip(this.components);
@@ -546,7 +555,7 @@ namespace NUnit.Gui
             this.runFailedMenuItem,
             this.testMenuSeparator,
             this.stopRunMenuItem});
-            this.testMenu.Text = "&Test";
+            this.testMenu.Text = "&Tests";
             // 
             // runAllMenuItem
             // 
@@ -588,6 +597,7 @@ namespace NUnit.Gui
             this.assemblyDetailsMenuItem,
             this.saveXmlResultsMenuItem,
             this.exceptionDetailsMenuItem,
+            this.openLogDirectoryMenuItem,
             this.toolsMenuSeparator1,
             this.settingsMenuItem,
             this.toolsMenuSeparator2,
@@ -613,25 +623,31 @@ namespace NUnit.Gui
             this.exceptionDetailsMenuItem.Text = "&Exception Details...";
             this.exceptionDetailsMenuItem.Click += new System.EventHandler(this.exceptionDetailsMenuItem_Click);
             // 
+            // openLogDirectoryMenuItem
+            // 
+            this.openLogDirectoryMenuItem.Index = 3;
+            this.openLogDirectoryMenuItem.Text = "Open &Log Directory...";
+            this.openLogDirectoryMenuItem.Click += new System.EventHandler(this.openLogDirectoryMenuItem_Click);
+            // 
             // toolsMenuSeparator1
             // 
-            this.toolsMenuSeparator1.Index = 3;
+            this.toolsMenuSeparator1.Index = 4;
             this.toolsMenuSeparator1.Text = "-";
             // 
             // settingsMenuItem
             // 
-            this.settingsMenuItem.Index = 4;
+            this.settingsMenuItem.Index = 5;
             this.settingsMenuItem.Text = "&Settings...";
             this.settingsMenuItem.Click += new System.EventHandler(this.settingsMenuItem_Click);
             // 
             // toolsMenuSeparator2
             // 
-            this.toolsMenuSeparator2.Index = 5;
+            this.toolsMenuSeparator2.Index = 6;
             this.toolsMenuSeparator2.Text = "-";
             // 
             // addinInfoMenuItem
             // 
-            this.addinInfoMenuItem.Index = 6;
+            this.addinInfoMenuItem.Index = 7;
             this.addinInfoMenuItem.Text = "Addins...";
             this.addinInfoMenuItem.Click += new System.EventHandler(this.addinInfoMenuItem_Click);
             // 
@@ -684,10 +700,10 @@ namespace NUnit.Gui
             // 
             // groupBox1
             // 
+            this.groupBox1.Controls.Add(this.suiteName);
             this.groupBox1.Controls.Add(this.runCount);
             this.groupBox1.Controls.Add(this.stopButton);
             this.groupBox1.Controls.Add(this.runButton);
-            this.groupBox1.Controls.Add(this.suiteName);
             this.groupBox1.Controls.Add(this.progressBar);
             this.groupBox1.Dock = System.Windows.Forms.DockStyle.Top;
             this.groupBox1.Location = new System.Drawing.Point(0, 0);
@@ -696,43 +712,45 @@ namespace NUnit.Gui
             this.groupBox1.TabIndex = 0;
             this.groupBox1.TabStop = false;
             // 
+            // suiteName
+            // 
+            this.suiteName.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
+                        | System.Windows.Forms.AnchorStyles.Right)));
+            this.suiteName.AutoEllipsis = true;
+            this.suiteName.Location = new System.Drawing.Point(145, 21);
+            this.suiteName.Name = "suiteName";
+            this.suiteName.Size = new System.Drawing.Size(343, 23);
+            this.suiteName.TabIndex = 1;
+            // 
             // runCount
             // 
             this.runCount.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                         | System.Windows.Forms.AnchorStyles.Right)));
-            this.runCount.Location = new System.Drawing.Point(8, 96);
+            this.runCount.AutoEllipsis = true;
+            this.runCount.Location = new System.Drawing.Point(8, 89);
             this.runCount.Name = "runCount";
-            this.runCount.Size = new System.Drawing.Size(480, 16);
+            this.runCount.Size = new System.Drawing.Size(480, 21);
             this.runCount.TabIndex = 5;
-            this.runCount.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
             // 
             // stopButton
             // 
-            this.stopButton.Location = new System.Drawing.Point(96, 18);
+            this.stopButton.AutoSize = true;
+            this.stopButton.Location = new System.Drawing.Point(75, 16);
             this.stopButton.Name = "stopButton";
-            this.stopButton.Size = new System.Drawing.Size(96, 38);
+            this.stopButton.Size = new System.Drawing.Size(64, 31);
             this.stopButton.TabIndex = 4;
             this.stopButton.Text = "&Stop";
             this.stopButton.Click += new System.EventHandler(this.stopButton_Click);
             // 
             // runButton
             // 
-            this.runButton.Location = new System.Drawing.Point(8, 18);
+            this.runButton.Location = new System.Drawing.Point(8, 16);
             this.runButton.Name = "runButton";
-            this.runButton.Size = new System.Drawing.Size(88, 38);
+            this.runButton.Size = new System.Drawing.Size(64, 31);
             this.runButton.TabIndex = 3;
             this.runButton.Text = "&Run";
             this.runButton.Click += new System.EventHandler(this.runButton_Click);
             // 
-            // suiteName
-            // 
-            this.suiteName.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
-                        | System.Windows.Forms.AnchorStyles.Right)));
-            this.suiteName.Location = new System.Drawing.Point(215, 23);
-            this.suiteName.Name = "suiteName";
-            this.suiteName.Size = new System.Drawing.Size(273, 29);
-            this.suiteName.TabIndex = 2;
-            // 
             // progressBar
             // 
             this.progressBar.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
@@ -741,7 +759,7 @@ namespace NUnit.Gui
             this.progressBar.CausesValidation = false;
             this.progressBar.Enabled = false;
             this.progressBar.ForeColor = System.Drawing.SystemColors.Highlight;
-            this.progressBar.Location = new System.Drawing.Point(8, 60);
+            this.progressBar.Location = new System.Drawing.Point(8, 54);
             this.progressBar.Maximum = 100;
             this.progressBar.Minimum = 0;
             this.progressBar.Name = "progressBar";
@@ -797,6 +815,7 @@ namespace NUnit.Gui
             this.Closing += new System.ComponentModel.CancelEventHandler(this.NUnitForm_Closing);
             this.rightPanel.ResumeLayout(false);
             this.groupBox1.ResumeLayout(false);
+            this.groupBox1.PerformLayout();
             this.leftPanel.ResumeLayout(false);
             this.ResumeLayout(false);
 
@@ -804,9 +823,18 @@ namespace NUnit.Gui
 
 		#endregion
 
-		#region Properties used internally
+        #region Public Properties
 
-		private TestLoader _testLoader;
+        public NUnitPresenter Presenter
+        {
+            get { return presenter; }
+        }
+
+        #endregion
+
+        #region Properties used internally
+
+        private TestLoader _testLoader;
 		private TestLoader TestLoader
 		{
 			get
@@ -869,10 +897,17 @@ namespace NUnit.Gui
                 foreach (RuntimeFramework framework in frameworks)
                 {
                     MenuItem item = new MenuItem(framework.DisplayName);
-                    item.Checked = current.Matches(framework);
+                    item.Checked = current.Supports(framework);
                     item.Tag = framework;
                     item.Click += new EventHandler(runtimeFrameworkMenuItem_Click);
-                    item.Enabled = NUnitConfiguration.GetNUnitBinDirectory(framework.ClrVersion) != null;
+                    try
+                    {
+                        item.Enabled = TestLoader.CanReloadUnderRuntimeVersion(framework.ClrVersion);
+                    }
+                    catch
+                    {
+                        item.Enabled = false;
+                    }
                     runtimeMenuItem.MenuItems.Add(item);
                 }
             }
@@ -887,30 +922,27 @@ namespace NUnit.Gui
 
 		private void newMenuItem_Click(object sender, System.EventArgs e)
 		{
-			if ( IsProjectLoaded )
-				TestLoaderUI.CloseProject( this );
-
-			TestLoaderUI.NewProject( this );
+            presenter.NewProject();
 		}
 
 		private void openMenuItem_Click(object sender, System.EventArgs e)
 		{
-			TestLoaderUI.OpenProject( this );
+            presenter.OpenProject();
 		}
 
 		private void closeMenuItem_Click(object sender, System.EventArgs e)
 		{
-			TestLoaderUI.CloseProject( this );
+            presenter.CloseProject();
 		}
 
 		private void saveMenuItem_Click(object sender, System.EventArgs e)
 		{
-			TestLoaderUI.SaveProject( this );
+            presenter.SaveProject();
 		}
 
 		private void saveAsMenuItem_Click(object sender, System.EventArgs e)
 		{
-			TestLoaderUI.SaveProjectAs( this );
+            presenter.SaveProjectAs();
             SetTitleBar(TestProject.Name);
 		}
 
@@ -921,17 +953,7 @@ namespace NUnit.Gui
 
 		private void reloadProjectMenuItem_Click(object sender, System.EventArgs e)
 		{
-            bool wrapper = TestProject.IsAssemblyWrapper;
-            string projectPath = TestProject.ProjectPath;
-            string activeConfigName = TestProject.ActiveConfigName;
-
-            // Unload first to avoid message asking about saving
-            TestLoader.UnloadProject();
-
-			if ( wrapper )
-				TestLoaderUI.OpenProject( this, projectPath );
-			else
-				TestLoaderUI.OpenProject( this, projectPath, activeConfigName, null );
+            presenter.ReloadProject();
 		}
 
 		private void reloadTestsMenuItem_Click(object sender, System.EventArgs e)
@@ -1181,25 +1203,19 @@ namespace NUnit.Gui
 
 		private void addAssemblyMenuItem_Click(object sender, System.EventArgs e)
 		{
-            TestLoaderUI.AddAssembly(this);
+            presenter.AddAssembly();
             LoadOrReloadTestAsNeeded();
 		}
 
 		private void addVSProjectMenuItem_Click(object sender, System.EventArgs e)
 		{
-            TestLoaderUI.AddVSProject(this);
+            presenter.AddVSProject();
             LoadOrReloadTestAsNeeded();
 		}
 
 		private void editProjectMenuItem_Click(object sender, System.EventArgs e)
 		{
-			using ( ProjectEditor editor = new ProjectEditor( TestProject ) )
-			{
-				this.Site.Container.Add( editor );
-				editor.ShowDialog( this );
-			}
-
-            LoadOrReloadTestAsNeeded();
+            presenter.EditProject();
         }
 
         private void LoadOrReloadTestAsNeeded()
@@ -1256,7 +1272,7 @@ namespace NUnit.Gui
 
 		private void saveXmlResultsMenuItem_Click(object sender, System.EventArgs e)
 		{
-			TestLoaderUI.SaveLastResult( this );
+            presenter.SaveLastResult();
 		}
 
 		private void exceptionDetailsMenuItem_Click(object sender, System.EventArgs e)
@@ -1284,6 +1300,14 @@ namespace NUnit.Gui
 			dlg.ShowDialog();
 		}
 
+        private void openLogDirectoryMenuItem_Click(object sender, EventArgs e)
+        {
+            if (!Directory.Exists(NUnitConfiguration.LogDirectory))
+                Directory.CreateDirectory(NUnitConfiguration.LogDirectory);
+
+            System.Diagnostics.Process.Start(NUnitConfiguration.LogDirectory);
+        }
+
 		#endregion
 
 		#region Help Menu
@@ -1337,17 +1361,21 @@ namespace NUnit.Gui
 				this.Invalidate();
 				this.Update();
 
+                // Set Capture options for the TestLoader
+                TestLoader.IsTracingEnabled = resultTabs.IsTracingEnabled;
+                TestLoader.LoggingThreshold = resultTabs.MaximumLogLevel;
+
 				// Load test specified on command line or
 				// the most recent one if options call for it
 				if ( guiOptions.ParameterCount != 0 )
-					TestLoaderUI.OpenProject( this, (string)guiOptions.Parameters[0], guiOptions.config, guiOptions.fixture );
+                    presenter.OpenProject((string)guiOptions.Parameters[0], guiOptions.config, guiOptions.fixture);
 				else if( userSettings.GetSetting( "Options.LoadLastProject", true ) && !guiOptions.noload )
 				{
 					foreach( RecentFileEntry entry in recentFilesService.Entries )
 					{
 						if ( entry != null && entry.Exists && entry.IsCompatibleCLRVersion )
 						{
-							TestLoaderUI.OpenProject( this, entry.Path, guiOptions.config, guiOptions.fixture );
+                            presenter.OpenProject(entry.Path, guiOptions.config, guiOptions.fixture);
 							break;
 						}
 					}
@@ -1378,10 +1406,10 @@ namespace NUnit.Gui
 						try
 						{
 							Services.UserSettings.SaveSetting("Options.TestLoader.ReloadOnRun", false);
-							if ( guiOptions.runselected )
-								testTree.RunSelectedTests();
-							else
-								TestLoader.RunTests();
+                            if (guiOptions.runselected)
+                                testTree.RunSelectedTests();
+                            else
+                                testTree.RunAllTests(false);
 						}
 						finally
 						{
@@ -1541,7 +1569,7 @@ namespace NUnit.Gui
 		{
 			if ( IsTestRunning )
 			{
-				DialogResult dialogResult = UserMessage.Ask( 
+                DialogResult dialogResult = MessageDisplay.Ask( 
 					"A test is running, do you want to stop the test and exit?" );
 
 				if ( dialogResult == DialogResult.No )
@@ -1550,8 +1578,8 @@ namespace NUnit.Gui
 					TestLoader.CancelTestRun();
 			}
 
-			if ( !e.Cancel && IsProjectLoaded && 
-				TestLoaderUI.CloseProject( this ) == DialogResult.Cancel )
+			if ( !e.Cancel && IsProjectLoaded &&
+                presenter.CloseProject() == DialogResult.Cancel)
 				e.Cancel = true;
 		}
 
@@ -1566,6 +1594,11 @@ namespace NUnit.Gui
 					else
 						displayMiniGui();
 			}
+            else if (args.SettingName.StartsWith("Gui.TextOutput.") && args.SettingName.EndsWith(".Content"))
+            {
+                TestLoader.IsTracingEnabled = resultTabs.IsTracingEnabled;
+                TestLoader.LoggingThreshold = resultTabs.MaximumLogLevel;
+            }
 		}
 		#endregion
 
@@ -1593,7 +1626,7 @@ namespace NUnit.Gui
 
 			if ( IsTestRunning )
 			{
-				DialogResult dialogResult = UserMessage.Ask( 
+                DialogResult dialogResult = MessageDisplay.Ask( 
 					"Do you want to cancel the running test?" );
 
 				if ( dialogResult == DialogResult.No )
@@ -1614,19 +1647,28 @@ namespace NUnit.Gui
 
 		#endregion
 
-		#region Event Handlers for Test Load and Unload
+		#region Event Handlers for Project Load and Unload
 
 		private void OnTestProjectLoaded( object sender, TestEventArgs e )
 		{
-			SetTitleBar( e.Name );
+            string projectPath = e.Name;
+
+			SetTitleBar( projectPath );
 			projectMenu.Visible = true;
 			runCount.Text = "";
+
+            // If this is an NUnit project, set up watcher
+            if (NUnitProject.IsNUnitProjectFile(projectPath) && File.Exists(projectPath))
+                presenter.WatchProject(projectPath);
 		}
 
-		private void OnTestProjectUnloading( object sender, TestEventArgs e )
+        private void OnTestProjectUnloading(object sender, TestEventArgs e)
 		{
-			if ( e.Name != null && File.Exists( e.Name ) )
-			{
+            // Remove any watcher
+            if (e.Name != null && File.Exists(e.Name))
+            {
+                presenter.RemoveWatcher();
+
 				Version version = Environment.Version;
 				foreach( TestAssemblyInfo info in TestLoader.AssemblyInfo )
 					if ( info.ImageRuntimeVersion < version )
@@ -1643,7 +1685,20 @@ namespace NUnit.Gui
 			runCount.Text = "";
 		}
 
-		private void OnTestLoadStarting( object sender, TestEventArgs e )
+        private void OnProjectLoadFailure(object sender, TestEventArgs e)
+        {
+            MessageDisplay.Error("Project Not Loaded", e.Exception);
+
+            recentFilesService.Remove(e.Name);
+
+            EnableRunCommand(IsProjectLoaded);
+        }
+
+        #endregion
+
+        #region Event Handlers for Test Load and Unload
+
+        private void OnTestLoadStarting(object sender, TestEventArgs e)
 		{
 			EnableRunCommand( false );
 			longOpDisplay = new LongRunningOperationDisplay( this, "Loading..." );
@@ -1678,7 +1733,7 @@ namespace NUnit.Gui
 				foreach( TestAssemblyInfo info in TestLoader.AssemblyInfo )
 					if ( info.TestFrameworks.Count > 0 ) return;
 
-				UserMessage.Display( "This assembly was not built with any known testing framework.", "Not a Test Assembly");
+                MessageDisplay.Error("This assembly was not built with any known testing framework.");
 			}
 		}
 
@@ -1715,17 +1770,8 @@ namespace NUnit.Gui
 			EnableRunCommand( true );
 		}
 
-		private void OnProjectLoadFailure( object sender, TestEventArgs e )
-		{
-			UserMessage.DisplayFailure( e.Exception, "Project Not Loaded" );
-
-			recentFilesService.Remove( e.Name );
-
-			EnableRunCommand( true );
-		}
-
 		/// <summary>
-		/// Event handler for assembly load faiulres. We do this via
+		/// Event handler for assembly load failures. We do this via
 		/// an event since some errors may occur asynchronously.
 		/// </summary>
 		private void OnTestLoadFailure( object sender, TestEventArgs e )
@@ -1736,14 +1782,17 @@ namespace NUnit.Gui
 				longOpDisplay = null;
 			}
 			
-			string message = null;
+			string message = e.Action == NUnit.Util.TestAction.TestReloadFailed
+                ? "Test reload failed!"
+                : "Test load failed!";
 			if ( e.Exception is BadImageFormatException )
-				message = string.Format(
+				message += string.Format(
+                    Environment.NewLine + Environment.NewLine +
 					@"You may be attempting to load an assembly built with a later version of the CLR than
 the version under which NUnit is currently running ({0}) or trying to load a 64-bit assembly into a 32-bit process.",
 					Environment.Version.ToString(3) );
 
-			UserMessage.DisplayFailure( e.Exception, message, "Assembly Not Loaded" );
+            MessageDisplay.Error(message, e.Exception);
 
 			if ( !IsTestLoaded )
 				OnTestUnloaded( sender, e );
@@ -1771,7 +1820,7 @@ the version under which NUnit is currently running ({0}) or trying to load a 64-
 			if ( e.Exception != null )
 			{
 				if ( ! ( e.Exception is System.Threading.ThreadAbortException ) )
-					UserMessage.DisplayFailure( e.Exception, "NUnit Test Run Failed" );
+                    MessageDisplay.Error("NUnit Test Run Failed", e.Exception);
 			}
 
 			ResultSummarizer summary = new ResultSummarizer( e.Result );
@@ -1779,7 +1828,16 @@ the version under which NUnit is currently running ({0}) or trying to load a 64-
                 "Passed: {0}   Failed: {1}   Errors: {2}   Inconclusive: {3}   Invalid: {4}   Ignored: {5}   Skipped: {6}   Time: {7}",
                 summary.Passed, summary.Failures, summary.Errors, summary.Inconclusive, summary.NotRunnable, summary.Ignored, summary.Skipped, summary.Time);
 
-			EnableRunCommand( true );
+            try
+            {
+                TestLoader.SaveLastResult("TestResult.xml");
+            }
+            catch (Exception ex)
+            {
+                log.Warning("Unable to save TestResult.xml\n{0}", ex.ToString());
+            }
+
+            EnableRunCommand(true);
 
             if (e.Result.ResultState == ResultState.Failure ||
                 e.Result.ResultState == ResultState.Error ||
