@@ -4,7 +4,7 @@
 // obtain a copy of the license at http://nunit.org
 // ****************************************************************
 
-#if NET_2_0
+#if CLR_2_0 || CLR_4_0
 using System;
 using NUnit.Framework;
 
