@@ -1,4 +1,10 @@
-﻿using System;
+﻿// ****************************************************************
+// Copyright 2012, Charlie Poole
+// This is free software licensed under the NUnit license. You may
+// obtain a copy of the license at http://nunit.org
+// ****************************************************************
+
+using System;
 using NUnit.Framework;
 
 namespace NUnit.TestData
